@@ -1,31 +1,55 @@
 package Mojolicious::Plugin::CHI;
 use Mojo::Base 'Mojolicious::Plugin';
+use Scalar::Util 'weaken';
 use CHI;
 
-our $VERSION = '0.06';
+our $VERSION = '0.08_1';
 
 # Register Plugin
 sub register {
   my ($plugin, $mojo, $param) = @_;
 
-  # Load parameter from Config file
+  # Load parameter from config file
   if (my $config_param = $mojo->config('CHI')) {
     $param = { %$config_param, %$param };
   };
 
   # Hash of cache handles
-  my $caches = {};
+  my $caches;
+
+  # Add 'chi_handles' attribute
+  # Necessary for multiple cache registrations
+  unless ($mojo->can('chi_handles')) {
+    $mojo->attr(
+      chi_handles => sub {
+	return ($caches //= {});
+      }
+    );
+  }
+
+  # Get caches from application
+  else {
+    $caches = $mojo->chi_handles;
+  };
+
 
   # Support namespaces
   my $ns = delete $param->{namespaces} // 1;
 
+  # Create log callback for CHI Logging
+  my $log = $mojo->log;
+  weaken $log;
+  my $log_ref = sub {
+    $log->warn( shift ) if defined $log;
+  };
+
   # Loop through all caches
   foreach my $name (keys %$param) {
     my $cache_param = $param->{$name};
 
     # Already exists
     if (exists $caches->{$name}) {
-      $mojo->log->warn("Multiple attempts to establish cache '$name'");
+      $mojo->log->warn(qq{Multiple attempts to establish cache "$name"});
       next;
     };
 
@@ -35,17 +59,25 @@ sub register {
     };
 
     # Get CHI handle
-    my $cache = CHI->new( %$cache_param );
+    my $cache = CHI->new(
+
+      # Set logging routines
+      on_get_error => $log_ref,
+      on_set_error => $log_ref,
+
+      %$cache_param
+    );
 
     # No succesful creation
-    unless ($cache) {
-      $mojo->log->warn("Unable to create cache handle '$name'");
-    };
+    $mojo->log->warn(qq{Unable to create cache handle "$name"}) unless $cache;
 
     # Store CHI handle
     $caches->{$name} = $cache;
   };
 
+  # Add 'chi' command
+  push @{$mojo->commands->namespaces}, __PACKAGE__;
+
 
   # Add 'chi' helper
   $mojo->helper(
@@ -56,11 +88,12 @@ sub register {
       my $cache = $caches->{$name};
 
       # Cache unknown
-      $mojo->log->warn("Unknown cache handle '$name'") unless $cache;
+      $c->app->log->warn(qq{Unknown cache handle "$name"}) unless $cache;
 
       # Return cache
       return $cache;
-    });
+    }
+  );
 };
 
 
@@ -75,7 +108,7 @@ __END__
 
 =head1 NAME
 
-Mojolicious::Plugin::CHI - Use CHI caches in Mojolicious
+Mojolicious::Plugin::CHI - Use CHI Caches in Mojolicious
 
 
 =head1 SYNOPSIS
@@ -151,7 +184,7 @@ associated with L<CHI> objects.
 
 All cache handles are qualified L<CHI> namespaces.
 You can omit this mapping by passing a C<namespaces>
-parameter with a false value.
+parameter with a C<false> value.
 
 The handles have to be unique, i.e.
 you can't have multiple different C<default> caches in mounted
@@ -160,6 +193,10 @@ applications using L<Mojolicious::Plugin::Mount>.
 All parameters can be set either on registration or
 as part of the configuration file with the key C<CHI>.
 
+Logging defaults to the application log, but can be
+overridden using L<on_get_error|CHI/on_get_error> and
+L<on_set_error|CHI/on_set_error>.
+
 
 =head1 HELPERS
 
@@ -176,21 +213,67 @@ If no cache handle name is given, a cache handle name
 C<default> is assumed.
 
 
+=head1 COMMANDS
+
+The following commands are available
+when the plugin is registered.
+
+=head2 chi list
+
+  perl app.pl chi list
+
+List all CHI caches associated with your application.
+
+
+=head2 chi purge
+
+  perl app.pl chi purge mycache
+
+Remove all expired entries from the cache namespace.
+
+
+=head2 chi clear
+
+  perl app.pl chi clear mycache
+
+Remove all entries from the cache namespace.
+
+
+=head2 chi expire
+
+  perl app.pl chi expire mykey
+  perl app.pl chi expire mycache mykey
+
+Set the expiration date of a key to the past.
+This does not necessarily delete the data,
+but makes it unavailable using L</get>.
+
+
+=head2 chi remove
+
+  perl app.pl chi remove mykey
+  perl app.pl chi remove mycache mykey
+
+Remove a key from the cache.
+
+
+
 =head1 DEPENDENCIES
 
 L<Mojolicious>,
 L<CHI>.
 
-B<Note:> L<CHI> has a lot of dependencies. It is
-thus not recommended to use this plugin in a CGI
-environment.
+B<Note:> Old versions of L<CHI> had a lot of dependencies.
+It was thus not recommended to use this plugin in a CGI
+environment. Since new versions of CHI use L<Moo> instead of
+L<Moose>, more use cases may be possible.
 
 
 =head1 CONTRIBUTORS
 
 L<Boris Däppen|https://github.com/borisdaeppen>
 
-L<reneeb|https://github.com/reneeb>
+L<Renée Bäcker|https://github.com/reneeb>
 
 
 =head1 AVAILABILITY
@@ -200,9 +283,9 @@ L<reneeb|https://github.com/reneeb>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-2013, L<Nils Diewald|http://nils-diewald.de/>.
+Copyright (C) 2012-2014, L<Nils Diewald|http://nils-diewald.de/>.
 
 This program is free software, you can redistribute it
-and/or modify it under the same terms as Perl.
+and/or modify it under the terms of the Artistic License version 2.0.
 
 =cut
